unit ReconU;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  LocItem = record
    start:  longint;
    count:  longint;
    end;

  Prefix = record
    RecCapacity:  longint;
    RecCount:     longint;
    MaxRec:       longint;
    FreeCapacity: longint;
    FreeCount:    longint;
    MaxFree:      longint;
    BlockSize:    longint;
    NextRid:      longint;
    end;

  LocList = array [0..0] of LocItem;
  LocListPtr = ^LocList;


type
  TfrmRecon = class(TForm)
    lblFileName: TLabel;
    dlgOpen: TOpenDialog;
    cbxBackup: TCheckBox;
    memCurrent: TMemo;
    lblCurrent: TLabel;
    lblNewRecCap: TLabel;
    lblNewFreeCap: TLabel;
    edtNewRecCap: TEdit;
    edtNewFreeCap: TEdit;
    btnRecon: TButton;
    btnReturn: TButton;
    procedure FormActivate(Sender: TObject);
    procedure btnReturnClick(Sender: TObject);
    procedure btnReconClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmRecon: TfrmRecon;

implementation

{$R *.DFM}

var
  TheFile,NewFile: file;
  TheFileName: string;
  P,NP: Prefix;
  LI: LocItem;
  OldOffset,NewOffset: integer;


procedure TfrmRecon.FormActivate(Sender: TObject);
begin
  memCurrent.Lines.Clear;
  dlgOpen.Execute;
  if dlgOpen.FileName <> '' then begin
    TheFileName := dlgOpen.FileName;
    lblFileName.Caption := 'Working with file: '+TheFileName;
    AssignFile(TheFile,TheFileName);
    reset(TheFile,1);
    BlockRead(TheFile,P,32);
    memCurrent.Lines.Add('Record Capacity is '
                        +IntToStr(P.RecCapacity));
    memCurrent.Lines.Add('Record Count is '
                        +IntToStr(P.RecCount));
    memCurrent.Lines.Add('Max records is '
                         +IntToStr(P.MaxRec));
    memCurrent.Lines.Add('Free Capacity is '
                         +IntToStr(P.FreeCapacity));
    memCurrent.Lines.Add('Free Count is '
                         +IntToStr(P.FreeCount));
    memCurrent.Lines.Add('MaxFree is '
                         +IntToStr(P.MaxFree));
    CloseFile(TheFile);
    end;
end;



procedure TfrmRecon.btnReconClick(Sender: TObject);

var
  OK: boolean;
  Buf: pointer;
  j: integer;
  NewBlock: integer;

begin
  OK := TRUE;
  NP := P;
  try
    NP.RecCapacity := StrToInt(edtNewRecCap.Text);
  except
    on EConvertError do begin
      ShowMessage('Error in new record capacity ');
      OK := FALSE;
      end;
    end;
  try
    NP.FreeCapacity := StrToInt(edtNewFreeCap.Text);
  except
    on EConvertError do begin
      ShowMessage('Error in new free capacity ');
      OK := FALSE;
      end;
    end;
  if not OK then exit;
  if NP.RecCapacity < P.MaxRec then begin
    ShowMessage('New record capacity must be at least as large '+
            'as old MaxRec');
    exit;
    end;
  NP.FreeCount := 0;
  NP.MaxFree := 0;

  reset(TheFile,1);
  AssignFile(NewFile,'QQQZZZ');
  rewrite(NewFile,1);
  BlockWrite(NewFile,NP,32);
  GetMem(Buf,8*(NP.RecCapacity+NP.FreeCapacity));
  fillchar(Buf^,8*(NP.RecCapacity+NP.FreeCapacity),0);
  BlockWrite(NewFile,Buf^,8*(NP.RecCapacity+NP.FreeCapacity));
  FreeMem(Buf,8*(NP.RecCapacity+NP.FreeCapacity));
  OldOffset := 32+8*(P.RecCapacity+P.FreeCapacity);
  NewOffset := 32+8*(NP.RecCapacity+NP.FreeCapacity);
  NewBlock := 0;

  for j := 1 to P.MaxRec do begin
    Seek(TheFile,24+8*j);
    BlockRead(TheFile,LI,8);
    if LI.count > 0 then begin
      GetMem(Buf,LI.count*P.BlockSize);
      Seek(TheFile,OldOffset+LI.start*P.BlockSize);
      BlockRead(TheFile,Buf^,LI.Count*P.BlockSize);
      LI.start := NewBlock;
      Seek(NewFile,24+8*j);
      BlockWrite(NewFile,LI,8);
      Seek(NewFile,NewOffset+LI.start*P.BlockSize);
      BlockWrite(NewFile,Buf^,LI.count*P.BlockSize);
      NewBlock := NewBlock + LI.count;
      FreeMem(Buf,LI.count*P.BlockSize);
      end
    else begin
      Seek(NewFile,24+8*j);
      BlockWrite(NewFile,LI,8);
      end;
    end;

  CloseFile(NewFile);
  CloseFile(TheFile);
  if cbxBackup.checked then begin
    DeleteFile(TheFileName+'.BAK');
    RenameFile(TheFileName,TheFileName+'.BAK');
    end
  else DeleteFile(TheFileName);
  RenameFile('QQQZZZ',TheFileName);
end;


procedure TfrmRecon.btnReturnClick(Sender: TObject);
begin
  Close;
end;

end.
